/* 
 * This file contains the definition and methods of the class
 * PacketQModule.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "inet.h"
#include "rtp.h"
#include "tclcl.h"
#include "pktbuf.h"
#include "codec/module.h"
#include "rtp/media-timer.h"

/*
 * The PacketQModule can be used to store a stream of 
 * packet buffers into a FIFO queue which can then be
 * dumped to another PacketModule derived object on command.
 * This class corresponds to the OTcl class "Module/PacketQ"
 */

class PacketQModule : public PacketModule {
public:
  PacketQModule() : PacketModule(), head_(0), tail_(0) {};
  void recv(pktbuf *pb);
  virtual int command(int argc, const char*const* argv);

protected:
  pktbuf *head_;
  pktbuf *tail_;
};

/*
 * Recv entry method of incoming packet buffers.
 * Packet buffers are simply stored in a queue until the
 * dump command is given on the OTcl interface at which
 * point the packets are forwarded to the target_'s recv
 * method.
 */

void
PacketQModule::recv(pktbuf *pb)
{
  if (head_ == 0) {
    head_ = tail_ = pb;
    pb->next = 0;
    return;
  }
  tail_->next = pb;
  pb->next = 0;
  tail_ = pb;
}

/*
 * OTcl interface to Module/PacketQ. Provides the
 * "dump" command which takes no arguments and causes
 * all currently queued packets to be forwarded to the
 * target.
 */

int 
PacketQModule::command(int argc, const char*const* argv)
{
  if (argc == 2) {
    if (strcmp(argv[1], "dump") == 0) {
      if (target_ != 0) {
	while(head_ != 0) {
	  pktbuf *pb;

	  pb = head_;
	  head_ = head_->next;
	  pb->next = 0;

	  target_->recv(pb);
	}
	tail_ = head_ = 0;
      } else {
	while(head_ != 0) {
	  pktbuf *pb;

	  pb = head_;
	  head_ = head_->next;
	  pb->next = 0;

	  pb->release();
	}
	tail_ = head_ = 0;
      }
      return TCL_OK;
    }
  }
  return (PacketModule::command(argc, argv));
}

/*
 * Binding between the C++ class PacketQModule and
 * the OTcl class Module/PacketQ.
 */

static class PacketQModuleClass : public TclClass {
public:
  PacketQModuleClass() : TclClass("Module/PacketQ") {}
  TclObject *create(int argc, const char*const* argv) {
    return (new PacketQModule());
  }
} sc_exp_pkt_q_;

